local addon, HB = ...

HB.SELLPRICE = 0

HB.RANKS = {
	{l="NOV", s=1, co=1, un=2, ra=5, ep=20, re=50},
	{l="SKL", s=80, co=1, un=3, ra=6, ep=22, re=55},
	{l="EXP", s=155, co=2, un=3, ra=7, ep=24, re=60},
	{l="MST", s=230, co=2, un=4, ra=8, ep=26, re=65},
	{l="GM", s=305, co=3, un=4, ra=9, ep=28, re=75},
	{l="SAV", s=380, co=3, un=5, ra=9, ep=30, re=75},
}

HB.ICONS = {
	[true] =	{i="img/tick_green.png", a=1.00},
	[false] =	{i="img/tick_empty.png", a=0.25}
}

HB.tRarityCols = {
	["co"] = {1.00,1.00,1.00,0.50},
	["un"] = {0.00,0.80,0.00,0.50},
	["ra"] = {0.15,0.51,0.99,0.50},
	["ep"] = {0.69,0.28,1.00,0.50},
	["re"] = {1.00,0.60,0.00,0.50}
}

HB.tArtiLookup = {
	["artifacts other"] = "oth",
	["artifact other"] = "oth",
	["artifact fishing"] = "fsh",
	["artifacts unstable"] = "uns",
	["artifact unstable"] = "uns",
	["artifacts twisted"] = "twi",
	["artifact twisted"] = "twi",
	["artifacts bounty"] = "bou",
	["artifact bounty"] = "bou",
	["artifacts normal"] = "nor",
	["artifact normal"] = "nor",	
	["common"] = "co",
	["uncommon"] = "un",
	["rare"] = "ra",
	["epic"] = "ep",
	["relic"] = "re"
}

HB.artifacts = {
	["common"] = 0,
	["uncommon"] = 0,
	["rare"] = 0,
	["epic"] = 0,
	["relic"] = 0,
}

HB.arti_category = {
	["artifacts other"] = "other",
	["artifact other"] = "other",
	["artifact fishing"] = "fishing",
	["artifacts unstable"] = "unstable",
	["artifact unstable"] = "unstable",
	["artifacts twisted"] = "twisted",
	["artifact twisted"] = "twisted",
	["artifacts bounty"] = "bounty",
	["artifact bounty"] = "bounty",
	["artifacts normal"] = "normal",
	["artifact normal"] = "normal",
}

HB.pid = Inspect.Unit.Lookup("player")
HB.cbtime = 0
HB.recalc = true
HB.debug = false
HB.previous = nil

local tConfig = {
	{ "co", "un", "ra", "ep", "re" },
	{ "oth", "nor", "twi", "uns", "bou", "fsh" },
}

local default_settings = {
	mmx=250, mmy=250, locked = false, size=32, alert=true, autohide=true, automove=true,
	cfgx=300, cfgy=300,
	am_settings = {
		["co_oth"] = false,		["co_nor"] = false,		["co_twi"] = false,		["co_uns"] = false,		["co_bou"] = false,		["co_fsh"] = false,
		["un_oth"] = false,		["un_nor"] = false,		["un_twi"] = false,		["un_uns"] = false,		["un_bou"] = false,		["un_fsh"] = false,
		["ra_oth"] = false,		["ra_nor"] = false,		["ra_twi"] = false,		["ra_uns"] = false,		["ra_bou"] = false,		["ra_fsh"] = false,
		["ep_oth"] = false,		["ep_nor"] = false,		["ep_twi"] = false,		["ep_uns"] = false,		["ep_bou"] = false,		["ep_fsh"] = false,
		["re_oth"] = false,		["re_nor"] = false,		["re_twi"] = false,		["re_uns"] = false,		["re_bou"] = false,		["re_fsh"] = false
	},
	mv_rarity = {
		["common"] = true,
		["uncommon"] = true,
		["rare"] = true,
		["epic"] = true,
		["relic"] = true,
		["other"] = true,
		["twisted"] = true,
		["normal"] = true,
		["unstable"] = false,
		["bounty"] = false,
	},
	rank = 5
}

local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}			
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

HB.ARTIFACT_SKIP = {
	-- Enrinke's Missing Artifacts
	["I451BEE60DFA747E5,0EEC425A57A1A4C9,,,,,,"] = true,
	["I1B7E66AA907C6890,268602BC7C8B34A4,,,,,,"] = true,
	-- Secrets of the Ascended
	["I338D0D8881DDAA33,E6926C4044EF158D,,,,,,"] = true,
	["I7DC69D2A85594D83,FDFF8FA5D225E29C,,,,,,"] = true,
	["I335EFA4442EF11E0,6CAB78E08DB6298E,,,,,,"] = true,
	-- Queen's Gambit
	["IFA0B7D383D5A6EE8,F65A826B25C863D1,,,,,,"] = true,
	["IFB9D866E7A4C1810,1FEB0514E6D43889,,,,,,"] = true,
	["IFB9D866D36E82D6F,2ACCDE6956AEA899,,,,,,"] = true,
	["IFB9D866C6F337151,0FFAB0BE715E3091,,,,,,"] = true,
	["IFB9D866BC3A4F0A0,69E3FFD4C4BDE30E,,,,,,"] = true,
	["IFB9D866A54FE3DFC,1A14C63210541DD9,,,,,,"] = true,
	["IFB9D8669C0A798CE,56AF2D6B72521F64,,,,,,"] = true,
	["IFB9D866837C0093A,44AC5308E6F4D4EC,,,,,,"] = true,
	["IFB9D8667765FD861,640E44406E3E4BAD,,,,,,"] = true,
	["IFB9D8666251A156F,2CBC7CD587253679,,,,,,"] = true,
	["IFB9D86659B2D8971,FDD5F55600913817,,,,,,"] = true,
	["IFB9D8664CFD7496A,08B7CEAB706BA827,,,,,,"] = true,
	-- Snake Eyes
	["I2AF3A9109417FF48,D6B32F56E3614C12,,,,,,"] = true,
	["I2B7D3C0A2E4F063A,0EC09CEEB871C6C2,,,,,,"] = true,
	["I2B7D3C0BD1F5FFB6,12E2070EDBF9F017,,,,,,"] = true,
	["I2B7D3C0C78E398F2,56482322F9FC71DB,,,,,,"] = true,
	["I2B7D3C0D346B5DB0,480CA38E377E5B3E,,,,,,"] = true,
	["I2B7D3C0EA01B0A3C,6681485FD6DDA4D1,,,,,,"] = true,
	["I2B7D3C0F7BFE0E40,68CCD3A571422DDE,,,,,,"] = true,
	["I2B7D3C108812648D,2C32EFBA8F44AFA2,,,,,,"] = true,
	["I2B7D3C1153915654,566025ADBA435F91,,,,,,"] = true,
	["I2B7D3C124E7FCD70,1ED3D8E59358000B,,,,,,"] = true,
	["I2B7D3C13612F0C4D,3CEA84A3559EB713,,,,,,"] = true,
	["I2B7D3C14A28FCC5A,3208AB4EE5C44703,,,,,,"] = true,
	["I2B7D3C15761B1879,46F96DE1CD99B7B2,,,,,,"] = true,
	["I2B7D3C16C28032B9,00D276D85F89AA62,,,,,,"] = true,
	["I2B7D3C17B3A1DD3A,12D44A9FEAE5EE3C,,,,,,"] = true,
	-- Seeing Dots
	["IFED743AA90CE3283,D669C446184C80B1,,,,,,"] = true,
	["IFA3157AE95BE4C02,71440A8BCB56F274,,,,,,"] = true,
	["IFA3157AD5392714B,5F4236C43FFA2B4C,,,,,,"] = true,
	["IFA3157AC8914845B,88E1972FF50867B8,,,,,,"] = true,
	["IFA3157AB25F27A42,219DD6430AFF2172,,,,,,"] = true,
	["IFA3157AAAF63F79B,2C7FAF987AD99182,,,,,,"] = true,
	["IFA3157A998598DD6,1738DA7092870EA7,,,,,,"] = true,
	["IFA3157A892BABD3F,369A485A19D0021A,,,,,,"] = true,
	-- A Knight's Tour
	["IFFD59DA676A5BE14,38404422B2A9A0BB,,,,,,"] = true,
	["IFB9049B49A1448E4,53EF147C209ABDE3,,,,,,"] = true,
	["IFB9049B35F9481D5,82A80B721AC39E4B,,,,,,"] = true,
	["IFB9049B2E5AD8051,6DB748DFB2EE2D9B,,,,,,"] = true,
	["IFB9049B195ECA934,7899223422C89DAD,,,,,,"] = true,
	["IFB9049B05B6E1D5B,5A827676608269F3,,,,,,"] = true,
	-- Little Black Book
	["IFEB6CF70BD5407B4,67F356BF358857F8,,,,,,"] = true,
	["IFEB6CF717C592130,5557FDEC8D4030A3,,,,,,"] = true,
	["IFEB6CF729FB79742,157918A0CE4C7367,,,,,,"] = true,
	["IFEB6CF73B2443090,05CEF9D337FC415F,,,,,,"] = true,
	["IFEB6CF7473A46169,FA399824EA8BA94A,,,,,,"] = true,
	["IFEB6CF755711E1A1,094D006D196FC6ED,,,,,,"] = true,
	["IFEB6CF763284F606,5D82F8427BC0135A,,,,,,"] = true,
	["IFEB6CF7751688956,30409D638CA28214,,,,,,"] = true,
	["IFEB6CF7889AD1AE9,61007B8EDD331599,,,,,,"] = true,
	["IFA8886E5BDA2A37C,55F1629873CD0084,,,,,,"] = true,
	-- Circuit Diagrams
	["IFB5B59E078C2D675,5903A7B592D89EA8,,,,,,"] = true,
	["IFA290B1CAE16A744,4D7FD19DC2EDE10A,,,,,,"] = true,
	["IFA290B1DBD652E79,F6F0AC14C232586A,,,,,,"] = true,
	["IFA290B1EB2CFD7ED,042DB235E7F1CF28,,,,,,"] = true,
	["IFA290B1FA3234732,28D64733C0C08490,,,,,,"] = true,
	["IFA290B20A77CB892,15892DF27060FDAF,,,,,,"] = true,
	["IFA290B219AF08228,0AA7549D00868D9F,,,,,,"] = true,
	["IFA290B2237F5AEB0,4C370E891564EBCC,,,,,,"] = true,
	["IFA290B23EFEB9429,3F7BDE885B8DE69A,,,,,,"] = true,
	["IFA290B24E877CF8F,5D928A469DD41A53,,,,,,"] = true,
	["IFA290B254FD2F48D,52B0B0F12DF9AA43,,,,,,"] = true,
	["IFA290B1BAE34AEC5,3B7D7A88379119E2,,,,,,"] = true,
	-- Bogling Star Signs
	["IFD8CC415A6A64F78,501282800441DF8B,,,,,,"] = true,
	["IFF4DA6363A602249,9B2E5D04C76A4026,,,,,,"] = true,
	["IFADBA74A93C50CBA,E9BA2300B2CA76E6,,,,,,"] = true,
	["IFC3F98ABA62DBC41,1F1CEF2D4C914081,,,,,,"] = true,
	["IFD5007DA7D0CB6DF,4AD39687ECEF7ADA,,,,,,"] = true,
	["IFC9D76AB74E6788C,EAFC534DBCE1A01E,,,,,,"] = true,
	["IFF6242499A457D80,2C1285E5B7E05BD9,,,,,,"] = true,
	["IFBC6875E4980B361,36B30E66CA571809,,,,,,"] = true,
	["IFC8A8F0C80B71C5F,EEA7C596315A7663,,,,,,"] = true,
	["IFF525FA356F9EE13,D0815952EA036F6F,,,,,,"] = true,
}

-- /script Inspect.Addon.Detail("Farmer").data.UI.btn.MouseDown = false

function HB.HandleButton()
	if Haberdash_Settings.locked or MINIMAPDOCKER then
		HB.UI.btn:SetTexture(addon.identifier, "img/btn_blue.png")
	else
		HB.UI.btn:SetTexture(addon.identifier, "img/btn_gray.png")
	end			
end

function HB.Resize()
	HB.UI.btn:SetHeight(Haberdash_Settings.size)
	HB.UI.btn:SetWidth(Haberdash_Settings.size)
end

function HB.UpdateTotals()
	local nco = HB.RANKS[Haberdash_Settings.rank].co
	local nun = HB.RANKS[Haberdash_Settings.rank].un
	local nra = HB.RANKS[Haberdash_Settings.rank].ra
	local nep = HB.RANKS[Haberdash_Settings.rank].ep
	local nre = HB.RANKS[Haberdash_Settings.rank].re

	HB.UI.dtls[1].tb:SetText(string.format("@%d=%d", nco, nco*HB.artifacts["common"]))
	HB.UI.dtls[2].tb:SetText(string.format("@%d=%d", nun, nun*HB.artifacts["uncommon"]))
	HB.UI.dtls[3].tb:SetText(string.format("@%d=%d", nra, nra*HB.artifacts["rare"]))
	HB.UI.dtls[4].tb:SetText(string.format("@%d=%d", nep, nep*HB.artifacts["epic"]))
	HB.UI.dtls[5].tb:SetText(string.format("@%d=%d", nre, nre*HB.artifacts["relic"]))
	
	local tot = (nco*HB.artifacts["common"]) + (nun*HB.artifacts["uncommon"]) + (nra*HB.artifacts["rare"]) + (nep*HB.artifacts["epic"]) + (nre*HB.artifacts["relic"])
	HB.UI.txt:SetText(tostring(tot))
end

function HB.BuildUI()
	HB.UI = {}
	HB.UI.icons = {}
	
	HB.UI.context = UI.CreateContext(addon.identifier)
	HB.UI.btn = UI.CreateFrame("Texture", "HB.UI.btn", HB.UI.context)
	HB.UI.btn:SetLayer(0)
	HB.UI.btn:SetHeight(10)
	HB.UI.btn:SetWidth(10)


	if not MINIMAPDOCKER then
		HB.UI.btn:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
			if not self.MouseDown then
				Haberdash_Settings.locked = not Haberdash_Settings.locked
				HB.UI.ctr:SetVisible(Haberdash_Settings.locked)
				HB.HandleButton()
			end
		end, "Event.UI.Input.Mouse.Right.Click")
		
		HB.UI.btn:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
			if not Haberdash_Settings.locked then
				self.MouseDown = false
				Haberdash_Settings.mmx = HB.UI.btn:GetLeft()
				Haberdash_Settings.mmy = HB.UI.btn:GetTop()
			end
		end, "Event.UI.Input.Mouse.Left.Up")
		
		HB.UI.btn:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
			if not Haberdash_Settings.locked then
				self.MouseDown = true
				local mouseData = Inspect.Mouse()
				self.sx = mouseData.x - HB.UI.btn:GetLeft()
				self.sy = mouseData.y - HB.UI.btn:GetTop()
			end
		end, "Event.UI.Input.Mouse.Left.Down")

		HB.UI.btn:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
			if self.MouseDown then
				local nx, ny
				local mouseData = Inspect.Mouse()
				nx = mouseData.x - self.sx
				ny = mouseData.y - self.sy
				HB.UI.btn:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)			
			end
		end, "Event.UI.Input.Mouse.Cursor.Move")

		HB.UI.btn:EventAttach(Event.UI.Input.Mouse.Wheel.Back, function(self, h)
			if Haberdash_Settings.size > 12 then
				Haberdash_Settings.size = Haberdash_Settings.size-1
				HB.Resize()
			end
		end, "Event.UI.Input.Mouse.Wheel.Back")
		
		HB.UI.btn:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, function(self, h)
			if Haberdash_Settings.size < 48 then
				Haberdash_Settings.size = Haberdash_Settings.size+1
				HB.Resize()
			end
		end, "Event.UI.Input.Mouse.Wheel.Forward")
	end

	HB.UI.btn:EventAttach(Event.UI.Layout.Size, function(self, h)
		local fs = HB.UI.btn:GetHeight()
		HB.UI.txt:SetFontSize(math.floor(fs/2))
	end, "Event.UI.Layout.Size")
	
	HB.UI.btn:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if Haberdash_Settings.locked or MINIMAPDOCKER then
			HB.UI.cfg:SetVisible(not HB.UI.cfg:GetVisible())
		end
	end, "Event.UI.Input.Mouse.Left.Click")
	
	HB.UI.btn:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self, h)
		if Haberdash_Settings.locked or MINIMAPDOCKER then
			if Haberdash_Settings.autohide then
				HB.UI.ctr:SetVisible(true)
			else
				HB.UI.ctr:SetVisible(not HB.UI.ctr:GetVisible())
			end
		end
	end, "Event.UI.Input.Mouse.Cursor.In")

	HB.UI.btn:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self, h)
		if (Haberdash_Settings.locked or MINIMAPDOCKER) and Haberdash_Settings.autohide then
			HB.UI.ctr:SetVisible(false)
		end
	end, "Event.UI.Input.Mouse.Cursor.Out")
	
	if MINIMAPDOCKER then
		HB.UI.txt = UI.CreateFrame("Text", "HB.UI.ctr", HB.UI.btn)
	else
		HB.UI.txt = UI.CreateFrame("Text", "HB.UI.ctr", HB.UI.context)
	end
	HB.UI.txt:SetPoint("CENTER", HB.UI.btn, "CENTER", 0,0)
	HB.UI.txt:SetLayer(10)
	HB.UI.txt:SetEffectGlow({strength=5, blurX=2, blurY=2})
	HB.UI.txt:SetText("0")
	
	HB.UI.ctr = UI.CreateFrame("Frame", "HB.UI.ctr", HB.UI.context)
	HB.UI.ctr:SetPoint("TOPLEFT", HB.UI.btn, "BOTTOMCENTER", 0,0)
	HB.UI.ctr:SetVisible(false)
	HB.UI.ctr:SetLayer(0)
	HB.UI.ctr:SetWidth(328)
	HB.UI.ctr:SetHeight(64)
	
	HB.UI.dtls = {}
	
	for x=1, 5 do
		local tx = UI.CreateFrame("Texture", "HB.UI.tx:"..x, HB.UI.ctr)
		tx:SetLayer(5)
		tx:SetWidth(64)
		tx:SetHeight(64)
		tx:SetTexture(addon.identifier, "img/ribbon.png")
		
		if x==1 then
			tx:SetPoint("TOPLEFT", HB.UI.ctr, "TOPLEFT", 0,0)
		else
			tx:SetPoint("TOPLEFT", HB.UI.dtls[x-1].tx, "TOPRIGHT", 0,0)
		end
		
		local tt = UI.CreateFrame("Text", "HB.UI.tt:"..x, HB.UI.ctr)
		tt:SetLayer(10)
		tt:SetPoint("BOTTOMCENTER", tx, "CENTER", 0,-2)
		tt:SetEffectGlow({strength=5, blurX=2, blurY=2})
		tt:SetFontSize(18)

		local tb = UI.CreateFrame("Text", "HB.UI.tb:"..x, HB.UI.ctr)
		tb:SetLayer(10)
		tb:SetPoint("TOPCENTER", tx, "CENTER", 0,2)
		tb:SetEffectGlow({strength=5, blurX=2, blurY=2})
		tb:SetFontSize(12)
		
		HB.UI.dtls[x] = { tx=tx, tt=tt, tb=tb }
	end
	
	HB.UI.dtls[1].tx:SetBackgroundColor(unpack(HB.tRarityCols.co))
	HB.UI.dtls[2].tx:SetBackgroundColor(unpack(HB.tRarityCols.un))
	HB.UI.dtls[3].tx:SetBackgroundColor(unpack(HB.tRarityCols.ra))
	HB.UI.dtls[4].tx:SetBackgroundColor(unpack(HB.tRarityCols.ep))
	HB.UI.dtls[5].tx:SetBackgroundColor(unpack(HB.tRarityCols.re))

	HB.UI.cfg = UI.CreateFrame("RiftWindow", "HB.UI.cfg", HB.UI.context)
	HB.UI.cfg:SetVisible(false)
	HB.UI.cfg:SetTitle("Haberdash Configuration")
	HB.UI.cfg:SetLayer(0)
	
	--print("HB.UI.cfg:GetHeight(): "..HB.UI.cfg:GetHeight())
	--print("HB.UI.cfg:GetWidth(): "..HB.UI.cfg:GetWidth())

	HB.UI.cfg:SetWidth(408)
		
	HB.UI.cfgclose = UI.CreateFrame("RiftButton", "HB.UI.cfgclose", HB.UI.cfg)
	HB.UI.cfgclose:SetSkin("close")
	HB.UI.cfgclose:SetPoint("TOPRIGHT", HB.UI.cfg, "TOPRIGHT", -6,6)
	HB.UI.cfgclose:SetLayer(5)
	HB.UI.cfgclose:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		HB.UI.cfg:SetVisible(false)
	end, "Event.UI.Input.Mouse.Left.Click")
	
	local cfgborder = HB.UI.cfg:GetBorder()
	
	cfgborder:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		self.MouseDown = true
		local mouseData = Inspect.Mouse()
		self.sx = mouseData.x - HB.UI.cfg:GetLeft()
		self.sy = mouseData.y - HB.UI.cfg:GetTop()
	end, "Event.UI.Input.Mouse.Left.Down")

	cfgborder:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			HB.UI.cfg:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	cfgborder:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		self.MouseDown = false
		Haberdash_Settings.cfgx = HB.UI.cfg:GetLeft()
		Haberdash_Settings.cfgy = HB.UI.cfg:GetTop()
	end, "Event.UI.Input.Mouse.Left.Up")

	local trl, trr, trt, trb
	trl, trt, trr, trb = HB.UI.cfg:GetTrimDimensions()
	
	local ww = HB.UI.cfg:GetWidth() - trl - trr
	
	HB.UI.h_options = UI.CreateFrame("Text", "HB.UI.h_options", HB.UI.cfg)
	HB.UI.h_options:SetText("Options:")
	HB.UI.h_options:SetPoint("TOPLEFT", HB.UI.cfg, "TOPLEFT", trl, trt)
	HB.UI.h_options:SetFontSize(18)
	
	HB.UI.l_options = UI.CreateFrame("Frame", "HB.UI.l_options", HB.UI.cfg)
	HB.UI.l_options:SetPoint("BOTTOMLEFT", HB.UI.h_options, "BOTTOMLEFT", 0,0)
	HB.UI.l_options:SetHeight(1)
	HB.UI.l_options:SetWidth(ww)
	HB.UI.l_options:SetBackgroundColor(1,0,0)

	HB.UI.recycle = UI.CreateFrame("Texture", "HB.UI.recycle", HB.UI.cfg)
	HB.UI.recycle:SetPoint("BOTTOMRIGHT", HB.UI.l_options, "TOPRIGHT", 0,0)
	HB.UI.recycle:SetWidth(24)
	HB.UI.recycle:SetHeight(24)
	HB.UI.recycle:SetTexture(addon.identifier, "img/recycle.png")
	HB.UI.recycle:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if HB.previous == nil then
			HB.previous = {}
			for kr,ar in ipairs(tConfig[1]) do
				for kc, ac in ipairs(tConfig[2]) do
					local cfg = ar.."_"..ac
					HB.previous[cfg] = Haberdash_Settings.am_settings[cfg]
					Haberdash_Settings.am_settings[cfg] = true
					HB.UI.arConfig[cfg].f:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.am_settings[cfg]].i)
				end
			end			
		else
			for kr,ar in ipairs(tConfig[1]) do
				for kc, ac in ipairs(tConfig[2]) do
					local cfg = ar.."_"..ac
					Haberdash_Settings.am_settings[cfg] = HB.previous[cfg]
					HB.UI.arConfig[cfg].f:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.am_settings[cfg]].i)
				end
			end
			HB.previous = nil
		end
		HB.RecalculateTotals()
	end, "Event.UI.Input.Mouse.Left.Click")
	
	HB.UI.cb_alert = UI.CreateFrame("Texture", "HB.UI.cb_alert", HB.UI.cfg)
	HB.UI.cb_alert:SetPoint("TOPLEFT", HB.UI.l_options, "BOTTOMLEFT", 0,2)
	HB.UI.cb_alert:SetWidth(24)
	HB.UI.cb_alert:SetHeight(24)
	HB.UI.cb_alert:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		Haberdash_Settings.alert = not Haberdash_Settings.alert
		HB.UI.cb_alert:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.alert].i)
	end, "Event.UI.Input.Mouse.Left.Click")
	
	HB.UI.t_alert = UI.CreateFrame("Text", "HB.UI.t_alert", HB.UI.cfg)
	HB.UI.t_alert:SetFontSize(16)
	HB.UI.t_alert:SetText("Alert when non-salvagable artifact received")
	HB.UI.t_alert:SetPoint("CENTERLEFT", HB.UI.cb_alert, "CENTERRIGHT", 2,0)

	HB.UI.cb_autohide = UI.CreateFrame("Texture", "HB.UI.cb_autohide", HB.UI.cfg)
	HB.UI.cb_autohide:SetPoint("TOPLEFT", HB.UI.cb_alert, "BOTTOMLEFT", 0,2)
	HB.UI.cb_autohide:SetWidth(24)
	HB.UI.cb_autohide:SetHeight(24)
	HB.UI.cb_autohide:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		Haberdash_Settings.autohide = not Haberdash_Settings.autohide
		HB.UI.cb_autohide:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.autohide].i)
	end, "Event.UI.Input.Mouse.Left.Click")

	HB.UI.t_autohide = UI.CreateFrame("Text", "HB.UI.t_autohide", HB.UI.cfg)
	HB.UI.t_autohide:SetFontSize(16)
	HB.UI.t_autohide:SetText("Autohide salvage summary")
	HB.UI.t_autohide:SetPoint("CENTERLEFT", HB.UI.cb_autohide, "CENTERRIGHT", 2,0)
	
	HB.UI.cb_automove = UI.CreateFrame("Texture", "HB.UI.cb_automove", HB.UI.cfg)
	HB.UI.cb_automove:SetPoint("TOPLEFT", HB.UI.cb_autohide, "BOTTOMLEFT", 0,2)
	HB.UI.cb_automove:SetWidth(24)
	HB.UI.cb_automove:SetHeight(24)
	HB.UI.cb_automove:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		Haberdash_Settings.automove = not Haberdash_Settings.automove
		HB.UI.cb_automove:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.automove].i)
	end, "Event.UI.Input.Mouse.Left.Click")

	HB.UI.t_automove = UI.CreateFrame("Text", "HB.UI.t_automove", HB.UI.cfg)
	HB.UI.t_automove:SetFontSize(16)
	HB.UI.t_automove:SetText("Automove artifacts into bag 1, slot 1")
	HB.UI.t_automove:SetPoint("CENTERLEFT", HB.UI.cb_automove, "CENTERRIGHT", 2,0)
	
	HB.UI.arConfig = {}

	HB.UI.t_normal = UI.CreateFrame("Text", "HB.UI.t_normal", HB.UI.cfg)
	HB.UI.t_normal:SetFontSize(16)
	HB.UI.t_normal:SetText("Normal")
	HB.UI.t_normal:SetPoint("TOPLEFT", HB.UI.t_automove, "BOTTOMLEFT", 2,0)

	HB.UI.t_twisted = UI.CreateFrame("Text", "HB.UI.t_twisted", HB.UI.cfg)
	HB.UI.t_twisted:SetFontSize(16)
	HB.UI.t_twisted:SetText("Twisted")
	HB.UI.t_twisted:SetPoint("TOPLEFT", HB.UI.t_normal, "BOTTOMLEFT", 0,2)

	HB.UI.t_unstable = UI.CreateFrame("Text", "HB.UI.t_unstable", HB.UI.cfg)
	HB.UI.t_unstable:SetFontSize(16)
	HB.UI.t_unstable:SetText("Unstable")
	HB.UI.t_unstable:SetPoint("TOPLEFT", HB.UI.t_twisted, "BOTTOMLEFT", 0,2)

	HB.UI.t_bounty = UI.CreateFrame("Text", "HB.UI.t_bounty", HB.UI.cfg)
	HB.UI.t_bounty:SetFontSize(16)
	HB.UI.t_bounty:SetText("Bounty")
	HB.UI.t_bounty:SetPoint("TOPLEFT", HB.UI.t_unstable, "BOTTOMLEFT", 0,2)

	HB.UI.t_fishing = UI.CreateFrame("Text", "HB.UI.t_fishing", HB.UI.cfg)
	HB.UI.t_fishing:SetFontSize(16)
	HB.UI.t_fishing:SetText("Fishing")
	HB.UI.t_fishing:SetPoint("TOPLEFT", HB.UI.t_bounty, "BOTTOMLEFT", 0,2)

	HB.UI.t_other = UI.CreateFrame("Text", "HB.UI.t_other", HB.UI.cfg)
	HB.UI.t_other:SetFontSize(16)
	HB.UI.t_other:SetText("Other")
	HB.UI.t_other:SetPoint("TOPLEFT", HB.UI.t_fishing, "BOTTOMLEFT", 0,2)
	
	for kr,ar in ipairs(tConfig[1]) do
		for kc, ac in ipairs(tConfig[2]) do
			local cfg = ar.."_"..ac
			
			local f = UI.CreateFrame("Texture", "HB.UI.arConfig:"..cfg, HB.UI.cfg)
			f:SetWidth(24)
			f:SetHeight(24)
			f:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
				Haberdash_Settings.am_settings[cfg] = not Haberdash_Settings.am_settings[cfg]
				HB.UI.arConfig[cfg].f:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.am_settings[cfg]].i)
				HB.RecalculateTotals()
			end, "Event.UI.Input.Mouse.Left.Click")
			f:SetBackgroundColor(unpack(HB.tRarityCols[ar]))
			
			HB.UI.arConfig[cfg] = {f=f}
		end
	end
	
	HB.UI.arConfig.co_nor.f:SetPoint("CENTERLEFT", HB.UI.t_normal, "CENTERLEFT", (ww/2),0)
	HB.UI.arConfig.un_nor.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_nor.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ra_nor.f:SetPoint("TOPLEFT", HB.UI.arConfig.un_nor.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ep_nor.f:SetPoint("TOPLEFT", HB.UI.arConfig.ra_nor.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.re_nor.f:SetPoint("TOPLEFT", HB.UI.arConfig.ep_nor.f, "TOPRIGHT", 2,0)

	HB.UI.arConfig.co_twi.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_nor.f, "BOTTOMLEFT", 0,2)
	HB.UI.arConfig.un_twi.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_twi.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ra_twi.f:SetPoint("TOPLEFT", HB.UI.arConfig.un_twi.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ep_twi.f:SetPoint("TOPLEFT", HB.UI.arConfig.ra_twi.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.re_twi.f:SetPoint("TOPLEFT", HB.UI.arConfig.ep_twi.f, "TOPRIGHT", 2,0)
	
	HB.UI.arConfig.co_uns.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_twi.f, "BOTTOMLEFT", 0,2)
	HB.UI.arConfig.un_uns.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_uns.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ra_uns.f:SetPoint("TOPLEFT", HB.UI.arConfig.un_uns.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ep_uns.f:SetPoint("TOPLEFT", HB.UI.arConfig.ra_uns.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.re_uns.f:SetPoint("TOPLEFT", HB.UI.arConfig.ep_uns.f, "TOPRIGHT", 2,0)

	HB.UI.arConfig.co_bou.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_uns.f, "BOTTOMLEFT", 0,2)
	HB.UI.arConfig.un_bou.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_bou.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ra_bou.f:SetPoint("TOPLEFT", HB.UI.arConfig.un_bou.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ep_bou.f:SetPoint("TOPLEFT", HB.UI.arConfig.ra_bou.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.re_bou.f:SetPoint("TOPLEFT", HB.UI.arConfig.ep_bou.f, "TOPRIGHT", 2,0)

	HB.UI.arConfig.co_fsh.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_bou.f, "BOTTOMLEFT", 0,2)
	HB.UI.arConfig.un_fsh.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_fsh.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ra_fsh.f:SetPoint("TOPLEFT", HB.UI.arConfig.un_fsh.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ep_fsh.f:SetPoint("TOPLEFT", HB.UI.arConfig.ra_fsh.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.re_fsh.f:SetPoint("TOPLEFT", HB.UI.arConfig.ep_fsh.f, "TOPRIGHT", 2,0)
	
	HB.UI.arConfig.co_oth.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_fsh.f, "BOTTOMLEFT", 0,2)
	HB.UI.arConfig.un_oth.f:SetPoint("TOPLEFT", HB.UI.arConfig.co_oth.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ra_oth.f:SetPoint("TOPLEFT", HB.UI.arConfig.un_oth.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.ep_oth.f:SetPoint("TOPLEFT", HB.UI.arConfig.ra_oth.f, "TOPRIGHT", 2,0)
	HB.UI.arConfig.re_oth.f:SetPoint("TOPLEFT", HB.UI.arConfig.ep_oth.f, "TOPRIGHT", 2,0)
	
	HB.UI.t_rank = UI.CreateFrame("Text", "HB.UI.t_rank", HB.UI.cfg)
	HB.UI.t_rank:SetFontSize(16)
	HB.UI.t_rank:SetText("Skill level for calculations")
	HB.UI.t_rank:SetPoint("TOPLEFT", HB.UI.t_other, "BOTTOMLEFT", 0,2)
	
	HB.UI.ranks = {}
	for x=1, #HB.RANKS do
		local cb = UI.CreateFrame("Texture", "HB.UI.ranks.cb:"..x, HB.UI.cfg)
		cb:SetWidth(24)
		cb:SetHeight(24)
		if x==1 then
			cb:SetPoint("TOPLEFT", HB.UI.t_rank, "BOTTOMLEFT", 0,2)
		else
			cb:SetPoint("TOPLEFT", HB.UI.ranks[x-1].cb, "BOTTOMLEFT", 0,2)
		end
		
		cb:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
			for r=1, #HB.RANKS do
				Haberdash_Settings.rank = x
				HB.UI.ranks[r].cb:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.rank == r].i)
				HB.UpdateTotals()
			end			
		end, "Event.UI.Input.Mouse.Left.Click")
		
		local t = UI.CreateFrame("Text", "HB.UI.ranks.t:"..x, HB.UI.cfg)
		t:SetFontSize(16)
		t:SetText(string.format("[%s] : %d / %d / %d / %d / %d", HB.RANKS[x].l,
			HB.RANKS[x].co, HB.RANKS[x].un, HB.RANKS[x].ra, HB.RANKS[x].ep, HB.RANKS[x].re))
		t:SetPoint("CENTERLEFT", cb, "CENTERRIGHT", 2,0)
		
		HB.UI.ranks[x] = {cb=cb, t=t}

	end
	
	HB.UI.cfg:SetHeight(trt+trb+ HB.UI.h_options:GetHeight() + ( (10+#HB.RANKS) * (2 + 24)))
end

function HB.RecalculateTotals(t)
	if HB.debug then print("Recalc totals()") end

	for k,v in pairs(HB.artifacts) do
		HB.artifacts[k] = 0
	end
	
	HB.tArtifactBreakdown = {}
	
	for k,v in pairs(Library.LibInventory.REFSLOTS["inventory"]) do
		local d = Library.LibInventory.GetSlotContent(k)
		if d and d.category and HB.arti_category[d.category] then
			if HB.ARTIFACT_SKIP[d.type] then
				if Haberdash_Settings.alert and t and t[k] then
					local pt, pb, ps = Utility.Item.Slot.Parse(k)
					print(string.format("*** %s [bag: %d, slot: %d] cannot be salvaged", d.name, pb, ps))
				end
			else
				local r = d.rarity or "common"
				local t = d.category
				local s = d.stack or 1
				local k = HB.tArtiLookup[r].."_"..HB.tArtiLookup[t]
				
				if HB.tArtifactBreakdown[k] == nil then
					HB.tArtifactBreakdown[k] = 0
				end
				
				HB.tArtifactBreakdown[k] = HB.tArtifactBreakdown[k] + s
				
				if Haberdash_Settings.am_settings[k] then
					HB.artifacts[r] = HB.artifacts[r]+s
				end
			end
		end
	end

	HB.UI.dtls[1].tt:SetText(string.format("#: %d", HB.artifacts["common"]))
	HB.UI.dtls[2].tt:SetText(string.format("#: %d", HB.artifacts["uncommon"]))
	HB.UI.dtls[3].tt:SetText(string.format("#: %d", HB.artifacts["rare"]))
	HB.UI.dtls[4].tt:SetText(string.format("#: %d", HB.artifacts["epic"]))
	HB.UI.dtls[5].tt:SetText(string.format("#: %d", HB.artifacts["relic"]))
	
	HB.UpdateTotals()
	HB.recalc = false
end

function HB.cb_libInvSLOT(t)
	local findarti = false
	
	if t["si01.001"] and Haberdash_Settings.automove then
		if t["si01.001"] == "nil" then
			if HB.debug then print("si01.001 = 'nil'. findarti = true") end
			findarti = true
		else
			local id = Library.LibInventory.GetSlotContent("si01.001")
			if id and id.category and HB.arti_category[id.category] == nil then
				if HB.debug then print("si01.001 ~= artifact ("..id.category.."). findarti = true") end
				findarti = true
			end			
		end
	end
	
	for k,v in pairs(t) do
		if v == "nil" or HB.arti_category[v.category] then
			HB.recalc = true
			break
		end
	end
	
	if HB.debug then print("findarti = "..tostring(findarti)) end
	if HB.recalc then
		HB.RecalculateTotals(t)
	end
	
	if findarti then
		if Inspect.Time.Real() - HB.cbtime < 2.5 then
			for k,v in pairs(Inspect.Item.Detail(Library.LibInventory.REFSLOTS["inventory"])) do
				if k ~= "si01.001" and v.category and HB.arti_category[v.category] and HB.ARTIFACT_SKIP[v.type] == nil then
					local rar = v.rarity or "common"
					local ctg = v.category
					local ack = HB.tArtiLookup[rar].."_"..HB.tArtiLookup[ctg]
					
					--print(k.." : "..v.name.." : "..ack)
					
					if Haberdash_Settings.am_settings[ack] then
						if HB.debug then print("moving arti from "..k.." to si01.001") end
						Command.Item.Move(k, "si01.001")
						findarti = false
						break
					else
						if HB.debug then  print(v.name.." ["..(v.rarity or "common").."] is not configured to move") end
					end
				end
			end
			if findarti then
				print("Checked all artifacts, none suitable")
				if Haberdash_Settings.custommsg then
					print(Haberdash_Settings.custommsg)
				end
				findarti = false
			end
		else
			if HB.debug then print("Update >2s") end
			findarti = false
		end
	end
	
	if findarti then
		print("Failed to find artifact, should not have reached here!")
		if Haberdash_Settings.custommsg then
			print(Haberdash_Settings.custommsg)
		end
		findarti = false
	end
end

function HB.Event_Unit_Availability_Full(h,t)
	for k,v in pairs(t) do
		if v == "player" then
			local a,b = next(Library.LibInventory.REFSLOTS["inventory"])
			Library.LibInventory.RegisterSlot(Library.LibInventory.REFSLOTS["inventory"], HB.cb_libInvSLOT)
			Command.Event.Detach(Event.Unit.Availability.Full, nil, nil, nil, addon.identifier)
		end
	end
end

function HB.Event_Addon_SavedVariables_Load_End(h,a)
	if a == addon.identifier then
		if Haberdash_Settings == nil then Haberdash_Settings = {} end
		MergeTable(Haberdash_Settings, default_settings)
		
		if MINIMAPDOCKER then
			MINIMAPDOCKER.Register(addon.identifier, HB.UI.btn)
		else
			HB.UI.btn:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Haberdash_Settings.mmx, Haberdash_Settings.mmy)
			HB.UI.btn:SetHeight(Haberdash_Settings.size)
			HB.UI.btn:SetWidth(Haberdash_Settings.size)
		end
		
		HB.UI.cfg:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Haberdash_Settings.cfgx,Haberdash_Settings.cfgy)
		HB.HandleButton()
		
		HB.UI.cb_alert:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.alert].i)
		HB.UI.cb_autohide:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.autohide].i)
		HB.UI.cb_automove:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.automove].i)
		
	for kr,ar in ipairs(tConfig[1]) do
		for kc, ac in ipairs(tConfig[2]) do
			local cfg = ar.."_"..ac
			HB.UI.arConfig[cfg].f:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.am_settings[cfg]].i)
		end
	end
		
		

--		HB.UI.cb_amcom:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.mv_rarity["common"]].i)
--		HB.UI.cb_amunc:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.mv_rarity["uncommon"]].i)
--		HB.UI.cb_amrar:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.mv_rarity["rare"]].i)
--		HB.UI.cb_amepi:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.mv_rarity["epic"]].i)
--		HB.UI.cb_amrel:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.mv_rarity["relic"]].i)
--		HB.UI.cb_amnor:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.mv_rarity["normal"]].i)
--		HB.UI.cb_amtwi:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.mv_rarity["twisted"]].i)
--		HB.UI.cb_amuns:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.mv_rarity["unstable"]].i)
--		HB.UI.cb_ambty:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.mv_rarity["bounty"]].i)
--		HB.UI.cb_amoth:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.mv_rarity["other"]].i)

		HB.UI.dtls[1].tx:SetAlpha(HB.ICONS[Haberdash_Settings.mv_rarity["common"]].a)
		HB.UI.dtls[2].tx:SetAlpha(HB.ICONS[Haberdash_Settings.mv_rarity["uncommon"]].a)
		HB.UI.dtls[3].tx:SetAlpha(HB.ICONS[Haberdash_Settings.mv_rarity["rare"]].a)
		HB.UI.dtls[4].tx:SetAlpha(HB.ICONS[Haberdash_Settings.mv_rarity["epic"]].a)
		HB.UI.dtls[5].tx:SetAlpha(HB.ICONS[Haberdash_Settings.mv_rarity["relic"]].a)
		
		for x=1, #HB.RANKS do
			HB.UI.ranks[x].cb:SetTexture(addon.identifier, HB.ICONS[Haberdash_Settings.rank == x].i)
		end
		
	end
end

function HB.Event_Unit_Castbar(h,t)
	if t[HB.pid] then
		local cb = Inspect.Unit.Castbar("player")
		if cb and cb.abilityName and cb.abilityName == "Salvage Artifacts" then
			HB.cbtime = Inspect.Time.Real()
		end
	end
end

function HB.FindSA()
	for k,v in pairs(Inspect.Ability.New.Detail(Inspect.Ability.New.List())) do
		if v.name == "Salvage Artifacts" then
			for ak, av in pairs(v) do
				print(ak.." : "..av)
			end
		end
	end
end

function HB.Command_Slash_Register(h,args)
	local r = {}
	for token in string.gmatch(args, "[^%s]+") do
		table.insert(r, token)
	end
	if r[1] == "findsa" then
		HB.FindSA()
	elseif r[1] == "debug" then		
		HB.debug = not HB.debug
		print("Debug = "..tostring(HB.debug))
	elseif r[1] == "sell" then
		local it, id = Inspect.Tooltip()
		if it == "item" and id then
			local d = Inspect.Item.Detail(id)
			if d and d.type == "I016AF7AA5BC9339B,48BD52D5A4C860D4,,,,,," then
				if HB.SELLPRICE > 0 then
					local tot = (d.stack or 1) * HB.SELLPRICE
					local ii = Inspect.Interaction()
					if ii.auction then
						print(string.format("Attempting to sell %d @ %d = %d", d.stack or 1, HB.SELLPRICE, tot))
						Command.Auction.Post(id, 48, tot, tot, true)
					else
						print("Nice try! Can't sell if auction not open!")
					end
				else
					print("No sell price configured for dream ribbons")
				end
			else
				print("Those are not dream ribbons!")
			end
		end
	elseif r[1] == "price" then
		if r[2] then
			HB.SELLPRICE = tonumber(r[2])
		end
		print("Current unit sell price: "..HB.SELLPRICE)
	elseif r[1] == "details" then
		HB.RecalculateTotals()
		print("Artifact Inventory summary:")
		print(string.format("NOR: c=%d, u=%d, r=%d, e=%d, R=%d", HB.tArtifactBreakdown.co_nor or 0, HB.tArtifactBreakdown.un_nor or 0, HB.tArtifactBreakdown.ra_nor or 0, HB.tArtifactBreakdown.ep_nor or 0, HB.tArtifactBreakdown.re_nor or 0))
		print(string.format("TWI: c=%d, u=%d, r=%d, e=%d, R=%d", HB.tArtifactBreakdown.co_twi or 0, HB.tArtifactBreakdown.un_twi or 0, HB.tArtifactBreakdown.ra_twi or 0, HB.tArtifactBreakdown.ep_twi or 0, HB.tArtifactBreakdown.re_twi or 0))
		print(string.format("UNS: c=%d, u=%d, r=%d, e=%d, R=%d", HB.tArtifactBreakdown.co_uns or 0, HB.tArtifactBreakdown.un_uns or 0, HB.tArtifactBreakdown.ra_uns or 0, HB.tArtifactBreakdown.ep_uns or 0, HB.tArtifactBreakdown.re_uns or 0))
		print(string.format("BOU: c=%d, u=%d, r=%d, e=%d, R=%d", HB.tArtifactBreakdown.co_nou or 0, HB.tArtifactBreakdown.un_nou or 0, HB.tArtifactBreakdown.ra_nou or 0, HB.tArtifactBreakdown.ep_nou or 0, HB.tArtifactBreakdown.re_nou or 0))
		print(string.format("FSH: c=%d, u=%d, r=%d, e=%d, R=%d", HB.tArtifactBreakdown.co_fsh or 0, HB.tArtifactBreakdown.un_fsh or 0, HB.tArtifactBreakdown.ra_fsh or 0, HB.tArtifactBreakdown.ep_fsh or 0, HB.tArtifactBreakdown.re_fsh or 0))
		print(string.format("OTH: c=%d, u=%d, r=%d, e=%d, R=%d", HB.tArtifactBreakdown.co_oth or 0, HB.tArtifactBreakdown.un_oth or 0, HB.tArtifactBreakdown.ra_oth or 0, HB.tArtifactBreakdown.ep_oth or 0, HB.tArtifactBreakdown.re_oth or 0))
	elseif r[1] == "test" then
		HB.cbtime = Inspect.Time.Real() -1
		HB.cb_libInvSLOT({["si01.001"] = "nil"})
	end
end

HB.BuildUI()

Command.Event.Attach(Command.Slash.Register("hbd"), HB.Command_Slash_Register, "Command.Slash.Register")
Command.Event.Attach(Event.Unit.Availability.Full, HB.Event_Unit_Availability_Full, "Event.Unit.Availability.Full")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, HB.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Event.Unit.Castbar, HB.Event_Unit_Castbar, "Event.Unit.Castbar")

print(string.format("v%s loaded.", addon.toc.Version))
